import chalk from 'chalk';
import type {LogLevel as EsbuildLogLevel} from 'esbuild';

export type LogLevel = 'silent' | 'error' | 'warn' | 'info' | 'verbose';

export function convertEsbuildLogLevel(level: EsbuildLogLevel): LogLevel {
  switch (level) {
    case 'debug':
      return 'verbose';
    case 'info':
      return 'info';
    case 'warning':
      return 'warn';
    case 'error':
      return 'error';
    case 'silent':
      return 'silent';
    default:
      return 'info';
  }
}

export interface ProcessRecord {
  file: string;
  status: 'success' | 'skipped' | 'failed';
  reason?: string;
}

export interface DetailedStats {
  total: number;
  success: number;
  skipped: number;
  failed: number;
  processedFiles: ProcessRecord[];
}

export interface Logger {
  info: (message: string, ...args: any[]) => void;
  warn: (message: string, ...args: any[]) => void;
  error: (message: string, ...args: any[]) => void;
  success: (message: string, ...args: any[]) => void;
  report: (stats: DetailedStats) => void;
  time: (label: string) => void;
  timeEnd: (label: string) => void;
  verbose: (message: string, ...args: any[]) => void;
  reportDetailed: (records: ProcessRecord[]) => void;
}

export interface LoggerOptions {
  level?: LogLevel;
}

class ConsoleLogger implements Logger {
  private readonly timers = new Map<string, number>();
  private readonly LOG_LEVELS: Record<LogLevel, number> = {
    silent: 0,
    error: 1,
    warn: 2,
    info: 3,
    verbose: 4,
  };

  constructor(
    private readonly prefix: string,
    private readonly level: LogLevel = 'info',
  ) {}

  private formatPrefix() {
    return chalk.cyan(`[${this.prefix}]`);
  }

  private formatTime(ms: number) {
    return chalk.gray(`(${ms}ms)`);
  }

  private shouldLog(messageLevel: LogLevel): boolean {
    if (this.level === 'silent') {
      return false;
    }

    return this.LOG_LEVELS[messageLevel] <= this.LOG_LEVELS[this.level];
  }

  info(message: string, ...args: any[]) {
    if (this.shouldLog('info')) {
      console.log(`${this.formatPrefix()} ${message}`, ...args);
    }
  }

  warn(message: string, ...args: any[]) {
    if (this.shouldLog('warn')) {
      console.warn(`${this.formatPrefix()} ${chalk.yellow('warning')} ${message}`, ...args);
    }
  }

  error(message: string, ...args: any[]) {
    if (this.shouldLog('error')) {
      console.error(`${this.formatPrefix()} ${chalk.red('❌')} ${message}`, ...args);
    }
  }

  success(message: string, ...args: any[]) {
    if (this.shouldLog('info')) {
      console.log(`${this.formatPrefix()} ${chalk.green('✓')} ${message}`, ...args);
    }
  }

  verbose(message: string, ...args: any[]) {
    if (this.shouldLog('verbose')) {
      console.log(`${this.formatPrefix()} ${chalk.gray(message)}`, ...args);
    }
  }

  time(label: string) {
    if (this.shouldLog('verbose')) {
      this.timers.set(label, performance.now());
    }
  }

  timeEnd(label: string) {
    if (this.shouldLog('verbose')) {
      const start = this.timers.get(label);
      if (start) {
        const duration = Math.round(performance.now() - start);
        console.log(`${this.formatPrefix()} ${label} ${this.formatTime(duration)}`);
        this.timers.delete(label);
      }
    }
  }

  report(stats: DetailedStats) {
    if (this.level === 'silent') return;

    const {total, success, skipped, failed} = stats;

    const summary = [
      `Total: ${total}`,
      `✓ Success: ${chalk.green(success)}`,
      skipped > 0 ? `⚠️ Skipped: ${chalk.yellow(skipped)}` : null,
      failed > 0 ? `❌ Failed: ${chalk.red(failed)}` : null,
    ]
      .filter(Boolean)
      .join('  ');

    console.log(`${this.formatPrefix()} Build Result: ${summary}`);

    if (stats.processedFiles?.length > 0) {
      this.reportDetailed(stats.processedFiles);
    }
  }

  reportDetailed(records: ProcessRecord[]) {
    if (!this.shouldLog('info')) return;

    const statusColors = {
      success: chalk.green,
      skipped: chalk.yellow,
      failed: chalk.red,
    };

    console.log(`\n${this.formatPrefix()} Detailed Processing Report:`);

    records.forEach(({file, status, reason}) => {
      const colorize = statusColors[status];
      const statusText = colorize(status.toUpperCase());
      const message = `${statusText} - ${file}${reason ? ` (${reason})` : ''}`;
      console.log(`${this.formatPrefix()} ${message}`);
    });
  }
}

export function createLogger(prefix: string, options: LoggerOptions = {}): Logger {
  return new ConsoleLogger(prefix, options.level ?? 'info');
}
